import json
from argparse import ArgumentParser
from collections.abc import Callable, Sequence
from pathlib import Path
from typing import Any, Optional

from vivaldi_feed_exporter.opml import opml

_PREFERENCE_FILENAME = "Preferences"
_FEEDS_GETTER: Callable[[dict[str, Any]], list[dict[str, str]]] = lambda p: p[
    "vivaldi"
]["rss"]["settings"]


def run():
    with open(parse_args().profile_folder / _PREFERENCE_FILENAME, encoding="utf8") as f:
        preferences = json.load(f)
    feeds = _FEEDS_GETTER(preferences)
    print(opml(feeds).to_xml())


def parse_args(args: Optional[Sequence[str]] = None):
    parser = ArgumentParser()
    parser.add_argument(
        "profile_folder", type=_existent_directory, metavar="<profile folder>"
    )
    return parser.parse_args(args)


def _existent_directory(value: str):
    path = Path(value)
    if path.exists() and path.is_dir():
        return path
    raise Exception



from collections.abc import Iterable

from opyml import OPML, Body, Head, Outline


def opml(feeds: Iterable[dict[str, str]]):
    return OPML(
        version="2.0",
        head=Head(title="Vivaldi Subscriptions"),
        body=Body(
            outlines=tuple(
                Outline(text=f["title"], xml_url=f["url"], title=f["title"])
                for f in feeds
            )
        ),
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vivaldi_feed_exporter']

package_data = \
{'': ['*']}

install_requires = \
['opyml>=0.1.1,<0.2.0']

entry_points = \
{'console_scripts': ['vivaldi-feed-exporter = vivaldi_feed_exporter.main:run']}

setup_kwargs = {
    'name': 'vivaldi-feed-exporter',
    'version': '0.1.0',
    'description': 'A command line tool to export feeds from Vivaldi as OPML',
    'long_description': "# vivaldi-feed-exporter\n\nSince [Vivaldi](https://vivaldi.com) does not currently have the ability to export feeds,\nI created a command line tool to generate [OPML](http://opml.org) based on a profile.\n\n*Disclaimer : It depends on Vivaldi's implementation, so it may stop working at any time*\n\n## Usage\n\n- Follow [this page](https://help.vivaldi.com/desktop/tools/import-and-export-browser-data/#Transfer_the_full_Vivaldi_browser_profile) \n  to identify your Vivaldi profile folder\n- Install `vivaldi-feed-exporter` via `pipx install vivaldi-feed-exporter`\n- Running `vivaldi-feed-exporter <profile folder path>` will output OPML to standard output\n\n## Mapping of Vivaldi feed properties to OPML `outline` element attributes\n\nVivaldi feed property |`outline` attribute\n-|-\ntitle|`text`, `title`\naddress|`xmlUrl`\n",
    'author': 'Akira Takahashi',
    'author_email': 'pretty.audience.64b9de@gizmotik.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/a-takahashi223/vivaldi-feed-exporter',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
